#include "params.h"
#include "gtksamba.h"
#include <unistd.h>

#define DEF_SMB_CONF  "/etc/smb.conf"
#define DEF_RESTART  "/etc/rc.d/init.d/smb restart"

static GtkWidget *pbox=NULL;
static GtkWidget *conf_entry=NULL;
static GtkWidget *restart_entry=NULL;



#ifdef HAVE_GNOME

void check_prefs(){
  gchar *path, *value;
  
  path=(char *)g_malloc((strlen(PACKAGE)+24)*sizeof(char));

 

  sprintf(path,"/%s/smblocal/conf",PACKAGE);
  value=gnome_config_get_string(path);
  if(value)
    g_free(value);
  else
    gnome_config_set_string(path,DEF_SMB_CONF);
  
  sprintf(path,"/%s/smblocal/restart",PACKAGE);
  value=gnome_config_get_string(path);
  if(value)
    g_free(value);
  else
    gnome_config_set_string(path,DEF_RESTART);

  g_free(path);

  return;
}

char *get_config_val(char *sect, char *name){
  char *path, *value;

  path=(char *)g_malloc((strlen(sect)+strlen(name)+strlen(PACKAGE)+4)
			* sizeof(char));
  sprintf(path,"/%s/%s/%s",PACKAGE,sect,name);
  value=gnome_config_get_string(path);
  g_free(path);
  return value;

}



void c_apply_prefs(GtkWidget *widget, gint page, gpointer  data){
  char *path, *value;
  GtkWidget *entry;

  path=(char *)g_malloc((strlen(PACKAGE)+24)*sizeof(char));


  value=gnome_file_entry_get_full_path(GNOME_FILE_ENTRY(conf_entry),
                                       FALSE);
  sprintf(path,"/%s/smblocal/conf",PACKAGE); 
  gnome_config_set_string(path,value);

  entry=gnome_entry_gtk_entry(GNOME_ENTRY(restart_entry));
  value=gtk_entry_get_text(GTK_ENTRY(entry));
  sprintf(path,"/%s/smblocal/restart",PACKAGE); 
  gnome_config_set_string(path,value);
  
  g_free(path);

  return;
}


void c_make_pref_box(GtkWidget *widget, gpointer  data){
  GtkWidget *entry;
  GtkWidget *label;
  GtkWidget *frame;
  GtkWidget *vbox;
  GtkWidget *hbox;
  char *value;

  pbox=gnome_property_box_new();
  gtk_signal_connect(GTK_OBJECT(pbox),"apply",
			    GTK_SIGNAL_FUNC(c_apply_prefs),
			    NULL);
  
  vbox = gtk_vbox_new(TRUE,0);
  gtk_widget_show (vbox);

  frame = gtk_frame_new ("Samba Configuration File");
  gtk_container_border_width (GTK_CONTAINER (frame), 10);
  gtk_box_pack_start (GTK_BOX (vbox), frame, FALSE, FALSE, 0);
  gtk_widget_show (frame);

  hbox = gtk_hbox_new(FALSE,0);
  gtk_container_border_width (GTK_CONTAINER (hbox), 10);
  gtk_container_add (GTK_CONTAINER (frame), hbox);
  gtk_widget_show (hbox);

  conf_entry = gnome_file_entry_new("local_conf","Samba Configuration File");
  value=get_config_val("smblocal","conf");
  entry=gnome_file_entry_gtk_entry(GNOME_FILE_ENTRY(conf_entry));
  gtk_entry_set_text (GTK_ENTRY (entry),value);
  
  g_free(value);
  gtk_signal_connect_object(GTK_OBJECT(entry),
			    "changed",
			    GTK_SIGNAL_FUNC(gnome_property_box_changed),
			    GTK_OBJECT(pbox));
  
  gtk_box_pack_start (GTK_BOX (hbox),conf_entry , 
		      TRUE, TRUE, 10);
  gtk_widget_show (conf_entry);  


  frame = gtk_frame_new ("Samba Restart Command");
  gtk_container_border_width (GTK_CONTAINER (frame), 10);
  gtk_box_pack_start (GTK_BOX (vbox), frame, FALSE, FALSE, 0);
  gtk_widget_show (frame);
  
  hbox = gtk_hbox_new(FALSE,0);
  gtk_container_border_width (GTK_CONTAINER (hbox), 10);
  gtk_container_add (GTK_CONTAINER (frame), hbox);
  gtk_widget_show (hbox);

  
  restart_entry = gnome_entry_new("local_restart");
  value=get_config_val("smblocal","restart");
  entry=gnome_entry_gtk_entry(GNOME_ENTRY(restart_entry));
  gtk_entry_set_text (GTK_ENTRY (entry),value);
  g_free(value);
  gtk_signal_connect_object(GTK_OBJECT(entry),"changed",
			    GTK_SIGNAL_FUNC(gnome_property_box_changed),
			    GTK_OBJECT(pbox));
  gtk_box_pack_start (GTK_BOX (hbox), restart_entry , 
		      TRUE, TRUE, 10);
  gtk_widget_show (restart_entry);
  
  label = gtk_label_new ("Local System");
  gtk_widget_show (label);
  gnome_property_box_append_page(GNOME_PROPERTY_BOX(pbox),vbox,label);

  frame = gtk_frame_new ("Remote Systems");
  gtk_container_border_width (GTK_CONTAINER (frame), 10);
  gtk_widget_show (frame);
  
  label = gtk_label_new ("This Feature Coming Soon");
  gtk_container_add (GTK_CONTAINER (frame), label);
  gtk_widget_show (label);
  
  label = gtk_label_new ("Remote Systems");
  gtk_widget_show (label);
  gnome_property_box_append_page(GNOME_PROPERTY_BOX(pbox), frame, label);

  gtk_widget_show(pbox);
  return;
}

#else

#define PREF_FILE "/.gtksambarc"

static GList *config=NULL;

typedef struct {
  char *section, *name, *value;
} Trio;

static Trio *make_trio(char *s,char *n, char *v){
  Trio *trio;

  if(!s && !n && !v)
    return NULL;

  trio = (Trio *)malloc(sizeof(Trio));
  trio->section = (char *)g_malloc((strlen(s)+1)*sizeof(char));
  trio->name = (char *)g_malloc((strlen(n)+1)*sizeof(char));
  trio->value = (char *)g_malloc((strlen(v)+1)*sizeof(char));
  strcpy(trio->section,s);
  strcpy(trio->name,n);
  strcpy(trio->value,v);

  return trio;

}

void destroy_trio(Trio *trio){
  if(trio){
    if(trio->section)
      g_free(trio->section);
    if(trio->name)
      g_free(trio->name);
    if(trio->value)
      g_free(trio->value);
    free(trio);
  }
  return;
}

void update_trio(Trio *trio,char *s,char *n, char *v){
  if(!s || !n || !v || !trio)
    return;
  trio->section = (char *)g_realloc(trio->section,(strlen(s)+1)*sizeof(char));
  trio->name = (char *)g_realloc(trio->name,(strlen(n)+1)*sizeof(char));
  trio->value = (char *)g_realloc(trio->value,(strlen(v)+1)*sizeof(char));
  strcpy(trio->section,s);
  strcpy(trio->name,n);
  strcpy(trio->value,v);

  return;

}


BOOL spref(char *section, void *data){
  strncpy((char *)(data),section,1024);
  return True;
}



BOOL ppref(char *pname, char *pvalue, void *data){
  Trio *trio;
  trio = make_trio((char *)(data),pname,pvalue);
  config = g_list_append(config,trio);
  return True;
}


void write_prefs(){
  char *homedir, *prefpath;
  FILE *prefout;
  char *last_section=NULL;
  Trio *trio;
  GList *outlist;

  outlist=config;

  homedir = getenv("HOME");
  prefpath = (char *)g_malloc((strlen(homedir)+strlen(PREF_FILE)+1)
			      * sizeof(char));
  sprintf(prefpath,"%s%s",homedir,PREF_FILE);
  
  if((prefout=fopen(prefpath,"w")) == NULL){
    make_user_mesg("Unable to open\nconfiguration file","Error");
    g_free(prefpath);
    return;
  }

  while(outlist){
    trio = (Trio *)(outlist->data);
    if(!last_section || strcmp(last_section,trio->section)){
      last_section=trio->section;
      fprintf(prefout,"[%s]\n",last_section);
    }
    
    fprintf(prefout,"%s = %s\n",trio->name,trio->value);
    outlist=outlist->next;
  }
  fclose(prefout);
  g_free(prefpath);
  return;
}

void check_prefs(){
  char *homedir, *prefpath; 
  char last_section[1024];
  Trio *trio;
  
  homedir = getenv("HOME");
  prefpath = (char *)g_malloc((strlen(homedir)+strlen(PREF_FILE)+1)
			    * sizeof(char));
  sprintf(prefpath,"%s%s",homedir,PREF_FILE);

  if(access(prefpath,R_OK)){
    trio = make_trio("smblocal","conf",DEF_SMB_CONF);
    config = g_list_append(config,trio);
    trio = make_trio("smblocal","restart",DEF_RESTART);
    config = g_list_append(config,trio);
    
    write_prefs();
  }
  else {
    last_section[0]='\0';
    pm_process(prefpath,spref,ppref,last_section);
  }  
  g_free(prefpath);
  return;
}

char *get_config_val(char *sect, char *name){
  GList *getlist;
  Trio *trio;
  char *valueout;

  getlist=config;

  while(getlist){
    trio = (Trio *)(getlist->data);
    if(!strcmp(sect,trio->section) &&
       !strcmp(name,trio->name)){
      valueout=(char *)g_malloc((strlen(trio->value)+1)*sizeof(char));
      strcpy(valueout,trio->value);
      return valueout;
    }
    getlist=getlist->next;
  }
  return NULL;
}

int set_config_val(char *sect, char *name, char *value){
  GList *getlist;
  Trio *trio;
  int n=0;

  getlist=config;

  while(getlist){
    trio = (Trio *)(getlist->data);
    if(!strcmp(sect,trio->section))
      break;
    n++;
    getlist=getlist->next;
  }

  if(!getlist){
    trio = make_trio(sect,name,value);
    config = g_list_append(config,trio);
    return -1;
  }

  while(getlist){
    trio = (Trio *)(getlist->data);
    if(strcmp(sect,trio->section))
      break;
    if(!strcmp(name,trio->name)){
      update_trio(trio,sect,name,value);
      return 0;
    }
    n++;
    getlist=getlist->next;
  }
  trio = make_trio(sect,name,value);
  if(getlist)
    config = g_list_insert(config,trio,n);
  else
    config = g_list_append(config,trio);
  return 1;
}
 
void c_pref_conf_browse_ok(GtkWidget *button, gpointer filew){
  
  gtk_entry_set_text (GTK_ENTRY (conf_entry),
		      gtk_file_selection_get_filename 
		      (GTK_FILE_SELECTION (filew)));
  gtk_widget_destroy(GTK_WIDGET(filew));
  return;
}


void c_pref_conf_browse(GtkWidget *button, gpointer data){
  
  GtkWidget *filew;
  filew = gtk_file_selection_new ("Default Config File");
  gtk_signal_connect_object(GTK_OBJECT (filew),
		       "delete_event",
		       GTK_SIGNAL_FUNC(gtk_widget_destroy),
		       GTK_OBJECT (filew));  
 
  gtk_signal_connect (GTK_OBJECT (GTK_FILE_SELECTION (filew)->ok_button),
		      "clicked", 
		      (GtkSignalFunc) c_pref_conf_browse_ok , 
		      filew);
    
  gtk_signal_connect_object(GTK_OBJECT (GTK_FILE_SELECTION(filew)->cancel_button),
			    "clicked",
			    GTK_SIGNAL_FUNC(gtk_widget_destroy),
			    GTK_OBJECT (filew));
  
  gtk_file_selection_set_filename(GTK_FILE_SELECTION(filew),
				  gtk_entry_get_text(GTK_ENTRY(conf_entry)));
  gtk_widget_show(GTK_WIDGET (filew));
  return;
}

  
void c_pref_apply(GtkWidget *widget, gpointer  data){
  char *value;
  
  value=gtk_entry_get_text(GTK_ENTRY(conf_entry));
  set_config_val("smblocal","conf",value);
  
  value=gtk_entry_get_text(GTK_ENTRY(restart_entry));
  set_config_val("smblocal","restart",value); 
  
  write_prefs();
  return;

}

void c_pref_ok(GtkWidget *widget, gpointer  data){
  c_pref_apply(widget,data);
  gtk_widget_hide(pbox);
  return;
}
  

void c_make_pref_box(GtkWidget *widget, gpointer  data){
  GtkWidget *notebook;
  GtkWidget *button;
  GtkWidget *label;
  GtkWidget *frame;
  GtkWidget *vbox;
  GtkWidget *hbox;
  char *value;

  if(!pbox){
    pbox = gtk_dialog_new();
    gtk_window_set_title(GTK_WINDOW(pbox),"GtkSamba Preferences");
    gtk_signal_connect_object (GTK_OBJECT (pbox), 
			       "delete_event",
			       GTK_SIGNAL_FUNC(gtk_widget_hide),
			       GTK_OBJECT (pbox));
    gtk_widget_set_usize(pbox, 500, 250);

    button = gtk_button_new_with_label("Ok");
    gtk_box_pack_start(GTK_BOX (GTK_DIALOG (pbox)->action_area), 
		       button, TRUE, TRUE, 10);
    gtk_signal_connect(GTK_OBJECT(button), "clicked",
		       GTK_SIGNAL_FUNC(c_pref_ok),
		       NULL);
    gtk_widget_show(button);

    button = gtk_button_new_with_label("Apply");
    gtk_box_pack_start(GTK_BOX (GTK_DIALOG (pbox)->action_area), 
		       button, TRUE, TRUE, 10);
    gtk_signal_connect(GTK_OBJECT(button), "clicked",
		       GTK_SIGNAL_FUNC(c_pref_apply),
		       NULL);
    gtk_widget_show(button);

    button = gtk_button_new_with_label("Cancel");
    gtk_box_pack_start(GTK_BOX (GTK_DIALOG (pbox)->action_area), 
		       button, TRUE, TRUE, 10);
    gtk_signal_connect_object(GTK_OBJECT(button), "clicked",
			      GTK_SIGNAL_FUNC(gtk_widget_hide),
			      GTK_OBJECT(pbox));
    gtk_widget_show(button);

    button = gtk_button_new_with_label("Help");
    gtk_box_pack_start(GTK_BOX (GTK_DIALOG (pbox)->action_area), 
		       button, TRUE, TRUE, 10);
    gtk_widget_show(button);

    notebook = gtk_notebook_new ();
    gtk_notebook_set_tab_pos (GTK_NOTEBOOK (notebook), GTK_POS_TOP);
    gtk_notebook_set_show_border(GTK_NOTEBOOK (notebook),TRUE);
    gtk_notebook_set_scrollable(GTK_NOTEBOOK (notebook),TRUE);
    gtk_box_pack_start (GTK_BOX (GTK_DIALOG (pbox)->vbox), 
            notebook, TRUE, TRUE, 0);
    gtk_widget_show(notebook);

    
    vbox = gtk_vbox_new(TRUE,0);
    gtk_widget_show (vbox);

    frame = gtk_frame_new ("Samba Configuration File");
    gtk_container_border_width (GTK_CONTAINER (frame), 10);
    gtk_box_pack_start (GTK_BOX (vbox), frame, FALSE, FALSE, 0);
    gtk_widget_show (frame);

    hbox = gtk_hbox_new(FALSE,0);
    gtk_container_border_width (GTK_CONTAINER (hbox), 10);
    gtk_container_add (GTK_CONTAINER (frame), hbox);
    gtk_widget_show (hbox);
    conf_entry = gtk_entry_new();
    value=get_config_val("smblocal","conf");
    gtk_entry_set_text (GTK_ENTRY (conf_entry),value);
    g_free(value);
    gtk_box_pack_start (GTK_BOX (hbox),conf_entry , 
			TRUE, TRUE, 10);
    gtk_widget_show (conf_entry);
    button = gtk_button_new_with_label("Browse...");
    gtk_signal_connect(GTK_OBJECT(button), "clicked",
		       GTK_SIGNAL_FUNC(c_pref_conf_browse),
		       NULL);
    gtk_box_pack_start(GTK_BOX (hbox), button, FALSE, FALSE, 0);
    gtk_widget_show(button);

    frame = gtk_frame_new ("Samba Restart Command");
    gtk_container_border_width (GTK_CONTAINER (frame), 10);
    gtk_box_pack_start (GTK_BOX (vbox), frame, FALSE, FALSE, 0);
    gtk_widget_show (frame);

    hbox = gtk_hbox_new(FALSE,0);
    gtk_container_border_width (GTK_CONTAINER (hbox), 10);
    gtk_container_add (GTK_CONTAINER (frame), hbox);
    gtk_widget_show (hbox);
    restart_entry = gtk_entry_new();
    value=get_config_val("smblocal","restart");
    gtk_entry_set_text (GTK_ENTRY (restart_entry),value);
    g_free(value);
    gtk_box_pack_start (GTK_BOX (hbox), restart_entry , 
			TRUE, TRUE, 10);
    gtk_widget_show (restart_entry);
    
    label = gtk_label_new ("Local System");
    gtk_widget_show (label);
    gtk_notebook_append_page (GTK_NOTEBOOK (notebook), vbox, label);


    frame = gtk_frame_new ("Remote Systems");
    gtk_container_border_width (GTK_CONTAINER (frame), 10);
    gtk_widget_show (frame);
    
    label = gtk_label_new ("This Feature Coming Soon");
    gtk_container_add (GTK_CONTAINER (frame), label);
    gtk_widget_show (label);
    
    label = gtk_label_new ("Remote Systems");
    gtk_widget_show (label);
    gtk_notebook_append_page (GTK_NOTEBOOK (notebook), frame, label);

    gtk_notebook_set_page(GTK_NOTEBOOK (notebook),0);
  }
  gtk_widget_show(pbox);
  return;
}

#endif
